@@ -707,13 +707,13 @@ class LUClusterRepairDiskSizes(NoHooksLU):
       for idx, disk in enumerate(self.cfg.GetInstanceDisks(instance.uuid)):
         per_node_disks[pnode].append((instance, idx, disk))
 
-    assert not (frozenset(per_node_disks.keys()) -
+    assert not (frozenset(per_node_disks) -
                 frozenset(self.owned_locks(locking.LEVEL_NODE_RES))), \
       "Not owning correct locks"
     assert not self.owned_locks(locking.LEVEL_NODE)
 
     es_flags = rpc.GetExclusiveStorageForNodes(self.cfg,
-                                               per_node_disks.keys())
+                                               list(per_node_disks))
 
     changed = []
     for node_uuid, dskl in per_node_disks.items():
@@ -746,7 +746,7 @@ class LUClusterRepairDiskSizes(NoHooksLU):
                           instance.name)
           continue
         (size, spindles) = dimensions
-        if not isinstance(size, (int, long)):
+        if not isinstance(size, int):
           self.LogWarning("Disk %d of instance %s did not return valid"
                           " size information, ignoring", idx, instance.name)
           continue
@@ -936,7 +936,7 @@ class LUClusterSetParams(LogicalUnit):
       try:
         utils.VerifyDictOptions(self.op.diskparams, constants.DISK_DT_DEFAULTS)
         CheckDiskAccessModeValidity(self.op.diskparams)
-      except errors.OpPrereqError, err:
+      except errors.OpPrereqError as err:
         raise errors.OpPrereqError("While verify diskparams options: %s" % err,
                                    errors.ECODE_INVAL)
 
@@ -1085,9 +1085,9 @@ class LUClusterSetParams(LogicalUnit):
       violations = set()
       for group in self.cfg.GetAllNodeGroupsInfo().values():
         instances = frozenset(
-          [inst for inst in all_instances
-           if compat.any(nuuid in group.members
-           for nuuid in self.cfg.GetInstanceNodes(inst.uuid))])
+          inst for inst in all_instances
+          if compat.any(nuuid in group.members
+          for nuuid in self.cfg.GetInstanceNodes(inst.uuid)))
         new_ipolicy = objects.FillIPolicy(self.new_ipolicy, group.ipolicy)
         ipol = masterd.instance.CalculateGroupIPolicy(cluster, group)
         new = ComputeNewInstanceViolations(ipol, new_ipolicy, instances,
@@ -1319,7 +1319,7 @@ class LUClusterSetParams(LogicalUnit):
           # check parameter syntax
           try:
             objects.NIC.CheckParameterSyntax(params_filled)
-          except errors.ConfigurationError, err:
+          except errors.ConfigurationError as err:
             nic_errors.append("Instance %s, nic/%d: %s" %
                               (instance.name, nic_idx, err))
 
@@ -1369,7 +1369,7 @@ class LUClusterSetParams(LogicalUnit):
 
       # Cleanup any OS that has an empty hypervisor parameter list, as we don't
       # need them in the cluster config anymore.
-      for os_name, hvs in self.new_os_hvp.items():
+      for os_name, hvs in list(self.new_os_hvp.items()):
         if not hvs:
           self.new_os_hvp.pop(os_name, None)
 
