@@ -31,8 +31,7 @@
 
 """
 
-import BaseHTTPServer
-import cgi
+import html
 import logging
 import os
 import socket
@@ -40,6 +39,8 @@ import time
 import signal
 import asyncore
 
+from http.server import BaseHTTPRequestHandler
+
 from ganeti import http
 from ganeti import utils
 from ganeti import netutils
@@ -86,12 +87,13 @@ class _HttpServerRequest(object):
   """Data structure for HTTP request on server side.
 
   """
-  def __init__(self, method, path, headers, body):
+  def __init__(self, method, path, headers, body, sock):
     # Request attributes
     self.request_method = method
     self.request_path = path
     self.request_headers = headers
     self.request_body = body
+    self.request_sock = sock
 
     # Response attributes
     self.resp_headers = {}
@@ -225,14 +227,15 @@ class _HttpClientToServerMessageReader(http.HttpMessageReader):
     return http.HttpClientToServerStartLine(method, path, version)
 
 
-def _HandleServerRequestInner(handler, req_msg):
+def _HandleServerRequestInner(handler, req_msg, reader):
   """Calls the handler function for the current request.
 
   """
   handler_context = _HttpServerRequest(req_msg.start_line.method,
                                        req_msg.start_line.path,
                                        req_msg.headers,
-                                       req_msg.body)
+                                       req_msg.body,
+                                       reader.sock)
 
   logging.debug("Handling request %r", handler_context)
 
@@ -246,14 +249,14 @@ def _HandleServerRequestInner(handler, req_msg):
     except (http.HttpException, errors.RapiTestResult,
             KeyboardInterrupt, SystemExit):
       raise
-    except Exception, err:
+    except Exception as err:
       logging.exception("Caught exception")
       raise http.HttpInternalServerError(message=str(err))
     except:
       logging.exception("Unknown exception")
       raise http.HttpInternalServerError(message="Unknown error")
 
-    if not isinstance(result, basestring):
+    if not isinstance(result, (str, bytes)):
       raise http.HttpError("Handler function didn't return string type")
 
     return (http.HTTP_OK, handler_context.resp_headers, result)
@@ -269,7 +272,7 @@ class HttpResponder(object):
   # Most web servers default to HTTP 0.9, i.e. don't send a status line.
   default_request_version = http.HTTP_0_9
 
-  responses = BaseHTTPServer.BaseHTTPRequestHandler.responses
+  responses = BaseHTTPRequestHandler.responses
 
   def __init__(self, handler):
     """Initializes this class.
@@ -308,8 +311,8 @@ class HttpResponder(object):
 
       (response_msg.start_line.code, response_msg.headers,
        response_msg.body) = \
-        _HandleServerRequestInner(self._handler, request_msg)
-    except http.HttpException, err:
+        _HandleServerRequestInner(self._handler, request_msg, req_msg_reader)
+    except http.HttpException as err:
       self._SetError(self.responses, self._handler, response_msg, err)
     else:
       # Only wait for client to close if we didn't have any exception.
@@ -338,7 +341,7 @@ class HttpResponder(object):
 
     values = {
       "code": err.code,
-      "message": cgi.escape(message),
+      "message": html.escape(message),
       "explain": longmsg,
       }
 
@@ -421,6 +424,8 @@ class HttpServerRequestExecutor(object):
           try:
             http.Handshake(sock, self.WRITE_TIMEOUT)
           except http.HttpSessionHandshakeUnexpectedEOF:
+            logging.debug("Unexpected EOF from %s:%s",
+                          client_addr[0], client_addr[1])
             # Ignore rest
             return
 
@@ -453,7 +458,7 @@ class HttpServerRequestExecutor(object):
       reader = _HttpClientToServerMessageReader(sock, msg, timeout)
     except http.HttpSocketTimeout:
       raise http.HttpError("Timeout while reading request")
-    except socket.error, err:
+    except socket.error as err:
       raise http.HttpError("Error reading request: %s" % err)
 
     return (msg, reader)
@@ -467,7 +472,7 @@ class HttpServerRequestExecutor(object):
       _HttpServerToClientMessageWriter(sock, req_msg, msg, timeout)
     except http.HttpSocketTimeout:
       raise http.HttpError("Timeout while sending response")
-    except socket.error, err:
+    except socket.error as err:
       raise http.HttpError("Error sending response: %s" % err)
 
 
@@ -475,9 +480,8 @@ class HttpServer(http.HttpBase, asyncore.dispatcher):
   """Generic HTTP server class
 
   """
-  MAX_CHILDREN = 20
 
-  def __init__(self, mainloop, local_address, port, handler,
+  def __init__(self, mainloop, local_address, port, max_clients, handler,
                ssl_params=None, ssl_verify_peer=False,
                request_executor_class=None, ssl_verify_callback=None):
     """Initializes the HTTP server
@@ -488,6 +492,11 @@ class HttpServer(http.HttpBase, asyncore.dispatcher):
     @param local_address: Local IP address to bind to
     @type port: int
     @param port: TCP port to listen on
+    @type max_clients: int
+    @param max_clients: maximum number of client connections
+        open simultaneously.
+    @type handler: HttpServerHandler
+    @param handler: Request handler object
     @type ssl_params: HttpSslParams
     @param ssl_params: SSL key and certificate
     @type ssl_verify_peer: bool
@@ -520,6 +529,7 @@ class HttpServer(http.HttpBase, asyncore.dispatcher):
     self._children = []
     self.set_socket(self.socket)
     self.accepting = True
+    self.max_clients = max_clients
     mainloop.RegisterSignal(self)
 
   def Start(self):
@@ -545,7 +555,7 @@ class HttpServer(http.HttpBase, asyncore.dispatcher):
     """
     if not quick:
       # Don't wait for other processes if it should be a quick check
-      while len(self._children) > self.MAX_CHILDREN:
+      while len(self._children) > self.max_clients:
         try:
           # Waiting without a timeout brings us into a potential DoS situation.
           # As soon as too many children run, we'll not respond to new
@@ -570,11 +580,23 @@ class HttpServer(http.HttpBase, asyncore.dispatcher):
 
     """
     # pylint: disable=W0212
+    t_start = time.time()
     (connection, client_addr) = self.socket.accept()
 
     self._CollectChildren(False)
 
-    pid = os.fork()
+    try:
+      pid = os.fork()
+    except OSError:
+      logging.exception("Failed to fork on request from %s:%s",
+                        client_addr[0], client_addr[1])
+      # Immediately close the connection. No SSL handshake has been done.
+      try:
+        connection.close()
+      except socket.error:
+        pass
+      return
+
     if pid == 0:
       # Child process
       try:
@@ -591,7 +613,13 @@ class HttpServer(http.HttpBase, asyncore.dispatcher):
         # In case the handler code uses temporary files
         utils.ResetTempfileModule()
 
+        t_setup = time.time()
         self.request_executor(self, self.handler, connection, client_addr)
+        t_end = time.time()
+        logging.debug("Request from %s:%s executed in: %.4f [setup: %.4f] "
+                      "[workers: %d]", client_addr[0], client_addr[1],
+                      t_end - t_start, t_setup - t_start, len(self._children))
+
       except Exception: # pylint: disable=W0703
         logging.exception("Error while handling request from %s:%s",
                           client_addr[0], client_addr[1])
