@@ -510,12 +510,12 @@ class QmpConnection(MonitorSocket):
       "mac": nic.mac,
     }
     # Note that hvinfo that _GenerateDeviceHVInfo() creates
-    # sould include *only* the driver, id, bus, and addr keys
+    # should include *only* the driver, id, bus, and addr keys
     arguments.update(self._filter_hvinfo(nic.hvinfo))
     if enable_mq:
       arguments.update({
         "mq": "on",
-        "vectors": (2 * virtio_net_queues + 1),
+        "vectors": (2 * virtio_net_queues + 2),
         })
     self.Execute("device_add", arguments)
 
@@ -702,6 +702,23 @@ class QmpConnection(MonitorSocket):
     if "netdev_add" not in self.supported_commands:
       _raise("netdev_add qmp command is not supported")
 
+  @_ensure_connection
+  def HasDynamicAutoReadOnly(self):
+    """Check if QEMU uses dynamic auto-read-only for block devices
+
+    Use QMP schema introspection (QEMU 2.5+) to check for the
+    dynamic-auto-read-only feature.
+    """
+    schema = self.Execute("query-qmp-schema")
+
+    # QEMU 4.0 did not have a feature flag, but has dynamic auto-read-only
+    # support.
+    if self.version[:2] == (4, 0):
+      return True
+
+    return any([x for x in schema
+                if "dynamic-auto-read-only" in x.get("features",[])])
+
   @_ensure_connection
   def SetMigrationParameters(self, max_bandwidth, downtime_limit):
     """Configute live migration parameters
