@@ -34,10 +34,11 @@
 
 import logging
 import errno
+import os
 import string # pylint: disable=W0402
 import shutil
 import time
-from cStringIO import StringIO
+from io import StringIO
 
 from ganeti import constants
 from ganeti import errors
@@ -148,12 +149,13 @@ def _ParseInstanceList(lines, include_node):
       raise errors.HypervisorError("Can't parse instance list,"
                                    " line: %s" % line)
     try:
+      # TODO: Cleanup this mess - introduce a namedtuple/dict/class
       data[1] = int(data[1])
       data[2] = int(data[2])
       data[3] = int(data[3])
       data[4] = _XenToHypervisorInstanceState(data[4])
       data[5] = float(data[5])
-    except (TypeError, ValueError), err:
+    except (TypeError, ValueError) as err:
       raise errors.HypervisorError("Can't parse instance list,"
                                    " line: %s, error: %s" % (line, err))
     except _InstanceCrashed:
@@ -168,6 +170,26 @@ def _ParseInstanceList(lines, include_node):
   return result
 
 
+def _InstanceDomID(info):
+  """Get instance domain ID from instance info tuple.
+  @type info: tuple
+  @param info: instance info as parsed by _ParseInstanceList()
+
+  @return: int, instance domain ID
+  """
+  return info[1]
+
+
+def _InstanceRuntime(info):
+  """Get instance runtime from instance info tuple.
+  @type info: tuple
+  @param info: instance info as parsed by _ParseInstanceList()
+
+  @return: float value of instance runtime
+  """
+  return info[5]
+
+
 def _GetAllInstanceList(fn, include_node, delays, timeout):
   """Return the list of instances including running and shutdown.
 
@@ -224,7 +246,7 @@ def _IsInstanceRunning(instance_info):
   ]
 
   def _RunningWithSuffix(suffix):
-    return map(lambda x: x + suffix, allowable_running_prefixes)
+    return [x + suffix for x in allowable_running_prefixes]
 
   # The shutdown suspend ("ss") state is encountered during migration, where
   # the instance is still considered to be running.
@@ -347,7 +369,7 @@ def _ParseNodeInfo(info):
     if len(fields) < 2:
       continue
 
-    (key, val) = map(lambda s: s.strip(), fields)
+    (key, val) = [s.strip() for s in fields]
 
     # Note: in Xen 3, memory has changed to total_memory
     if key in ("memory", "total_memory"):
@@ -368,7 +390,7 @@ def _ParseNodeInfo(info):
       xen_minor = int(val)
 
   if None not in [cores_per_socket, threads_per_core, nr_cpus]:
-    result["cpu_sockets"] = nr_cpus / (cores_per_socket * threads_per_core)
+    result["cpu_sockets"] = nr_cpus // (cores_per_socket * threads_per_core)
 
   if memory_free is not None:
     result["memory_free"] = memory_free
@@ -629,7 +651,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
     data.write("TAGS=\"%s\"\n" % r"\ ".join(instance.GetTags()))
     if nic.netinfo:
       netinfo = objects.Network.FromDict(nic.netinfo)
-      for k, v in netinfo.HooksDict().iteritems():
+      for k, v in netinfo.HooksDict().items():
         data.write("%s=\"%s\"\n" % (k, v))
 
     data.write("MAC=%s\n" % nic.mac)
@@ -645,7 +667,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
     try:
       utils.WriteFile(cfg_file, data=data.getvalue())
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise errors.HypervisorError("Cannot write Xen instance configuration"
                                    " file %s: %s" % (cfg_file, err))
 
@@ -715,7 +737,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
         if vif_type:
           nic_type_str = ", type=%s" % vif_type
       elif nic_type == constants.HT_NIC_PARAVIRTUAL:
-        nic_type_str = ", type=paravirtualized"
+        nic_type_str = ", type=%s" % constants.HT_HVM_VIF_VIF
       else:
         # parameter 'model' is only valid with type 'ioemu'
         nic_type_str = ", model=%s, type=%s" % \
@@ -764,7 +786,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
     cfg_file = self._ConfigFileName(instance_name)
     try:
       utils.WriteFile(cfg_file, data=data)
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise errors.HypervisorError("Cannot write Xen instance configuration"
                                    " file %s: %s" % (cfg_file, err))
 
@@ -776,7 +798,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
     try:
       file_content = utils.ReadFile(filename)
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise errors.HypervisorError("Failed to load Xen config file: %s" % err)
 
     return file_content
@@ -788,7 +810,7 @@ class XenHypervisor(hv_base.BaseHypervisor):
     utils.RemoveFile(self._ConfigFileName(instance_name))
     try:
       shutil.rmtree(self._InstanceNICDir(instance_name))
-    except OSError, err:
+    except OSError as err:
       if err.errno != errno.ENOENT:
         raise
 
@@ -842,12 +864,12 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
     """
     instance_list = self._GetInstanceList(instance_name == _DOM0_NAME, hvparams)
-    result = None
+
     for data in instance_list:
       if data[0] == instance_name:
-        result = data
-        break
-    return result
+        return data
+
+    return None
 
   def GetAllInstancesInfo(self, hvparams=None):
     """Get properties of all instances.
@@ -875,9 +897,39 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
     self._WriteConfigFile(instance.name, buf.getvalue())
 
+  def VerifyInstance(self, instance):
+    """Verify if running instance (configuration) is in correct state.
+
+    @type instance: L{objects.Instance}
+    @param instance: instance to verify
+
+    @return: bool, if instance in correct state
+    """
+    config_file = utils.PathJoin(self._cfgdir, "auto", instance.name)
+    return os.path.exists(config_file)
+
+  def RestoreInstance(self, instance, block_devices):
+    """Fixup running instance's state.
+
+    @type instance: L{objects.Instance}
+    @param instance: instance to restore
+    @type block_devices: list of tuples (disk_object, link_name, drive_uri)
+    @param block_devices: blockdevices assigned to this instance
+    """
+    startup_memory = self._InstanceStartupMemory(instance)
+    self._MakeConfigFile(instance, startup_memory, block_devices)
+
   def StartInstance(self, instance, block_devices, startup_paused):
     """Start an instance.
 
+    @type instance: L{objects.Instance}
+    @param instance: instance to start
+    @type block_devices: list of tuples (cfdev, rldev)
+      - cfdev: dict containing ganeti config disk part
+      - rldev: ganeti.block.bdev.BlockDev object
+    @param block_devices: blockdevices assigned to this instance
+    @type startup_paused: bool
+    @param startup_paused: if instance should be paused at startup
     """
     startup_memory = self._InstanceStartupMemory(instance)
 
@@ -932,16 +984,16 @@ class XenHypervisor(hv_base.BaseHypervisor):
                     or None for no timeout
 
     """
-    instance_info = self.GetInstanceInfo(name, hvparams=hvparams)
+    info = self.GetInstanceInfo(name, hvparams=hvparams)
 
-    if instance_info is None or _IsInstanceShutdown(instance_info[4]):
+    if info is None or hv_base.HvInstanceState.IsShutdown(info[4]):
       logging.info("Failed to shutdown instance %s, not running", name)
       return None
 
     return self._RunXen(["shutdown", "-w", name], hvparams, timeout)
 
   def _DestroyInstance(self, name, hvparams):
-    """Destroy an instance if the instance if the instance exists.
+    """Destroy an instance if the instance exists.
 
     @type name: string
     @param name: name of the instance to destroy
@@ -974,6 +1026,19 @@ class XenHypervisor(hv_base.BaseHypervisor):
     else:
       self._DestroyInstance(name, hvparams)
 
+  def _RenameInstance(self, old_name, new_name, hvparams):
+    """Rename an instance (domain).
+
+    @type old_name: string
+    @param old_name: current name of the instance
+    @type new_name: string
+    @param new_name: future (requested) name of the instace
+    @type hvparams: dict of string
+    @param hvparams: hypervisor parameters of the instance
+
+    """
+    return self._RunXen(["rename", old_name, new_name], hvparams)
+
   def _StopInstance(self, name, force, hvparams, timeout):
     """Stop an instance.
 
@@ -997,12 +1062,14 @@ class XenHypervisor(hv_base.BaseHypervisor):
       raise errors.HypervisorError("Failed to shutdown instance %s,"
                                    " not running" % name)
 
-    if force:
-      result = self._DestroyInstanceIfAlive(name, hvparams)
-    else:
+    if not force:
       self._ShutdownInstance(name, hvparams, timeout)
-      result = self._DestroyInstanceIfAlive(name, hvparams)
 
+    # TODO: Xen does always destroy the instnace after trying a gracefull
+    # shutdown. That means doing another attempt with force=True will not make
+    # any difference. This differs in behaviour from other hypervisors and
+    # should be cleaned up.
+    result = self._DestroyInstanceIfAlive(name, hvparams)
     if result is not None and result.failed and \
           self.GetInstanceInfo(name, hvparams=hvparams) is not None:
       raise errors.HypervisorError("Failed to stop instance %s: %s, %s" %
@@ -1032,7 +1099,8 @@ class XenHypervisor(hv_base.BaseHypervisor):
 
       # check if the domain ID has changed or the run time has decreased
       if (new_info is not None and
-          (new_info[1] != ini_info[1] or new_info[5] < ini_info[5])):
+          (_InstanceDomID(new_info) != _InstanceDomID(ini_info) or (
+              _InstanceRuntime(new_info) < _InstanceRuntime(ini_info)))):
         return
 
       raise utils.RetryAgain()
@@ -1206,24 +1274,39 @@ class XenHypervisor(hv_base.BaseHypervisor):
                                      " on port %d: check if port is available" %
                                      port)
 
-  def FinalizeMigrationDst(self, instance, info, success):
+  def FinalizeMigrationDst(self, instance, config, success):
     """Finalize an instance migration.
 
-    After a successful migration we write the xen config file.
-    We do nothing on a failure, as we did not change anything at accept time.
+    Write a config file if the instance is running on the destination node
+    regardles if we think the migration succeeded or not. This will cover cases,
+    when the migration succeeded but due to a timeout on the source node we
+    think it failed. If we think the migration failed and there is an unstarted
+    domain, clean it up.
 
     @type instance: L{objects.Instance}
     @param instance: instance whose migration is being finalized
-    @type info: string
-    @param info: content of the xen config file on the source node
+    @type config: string
+    @param config: content of the xen config file from the source node
     @type success: boolean
-    @param success: whether the migration was a success or a failure
+    @param success: whether the master node thinks the migration succeeded
 
     """
-    if success:
-      self._WriteConfigFile(instance.name, info)
-    elif self._UseMigrationDaemon(instance.hvparams):
-      XenHypervisor._KillMigrationDaemon(instance)
+
+    # We should recreate the config file if the domain is present and running,
+    # regardless if we think the migration succeeded or not.
+    info = self.GetInstanceInfo(instance.name, hvparams=instance.hvparams)
+    if info and _InstanceRuntime(info) != 0:
+      self._WriteConfigFile(instance.name, config)
+
+    if not success:
+      if self._UseMigrationDaemon(instance.hvparams):
+        XenHypervisor._KillMigrationDaemon(instance)
+
+      # Fix the common failure when the domain was created but never started:
+      # this happens if the memory transfer didn't complete and the instance
+      # is running on the source node.
+      if info and _InstanceRuntime(info) == 0:
+        self._DestroyInstance(instance.name, instance.hvparams)
 
   def MigrateInstance(self, _cluster_name, instance, target, live):
     """Migrate an instance to a target node.
@@ -1289,24 +1372,39 @@ class XenHypervisor(hv_base.BaseHypervisor):
       raise errors.HypervisorError("Failed to migrate instance %s: %s" %
                                    (instance_name, result.output))
 
-  def FinalizeMigrationSource(self, instance, success, live):
+  def FinalizeMigrationSource(self, instance, success, _):
     """Finalize the instance migration on the source node.
 
     @type instance: L{objects.Instance}
     @param instance: the instance that was migrated
     @type success: bool
-    @param success: whether the migration succeeded or not
-    @type live: bool
-    @param live: whether the user requested a live migration or not
+    @param success: whether the master thinks the migration succeeded
 
     """
     # pylint: disable=W0613
     if success:
-      # remove old xen file after migration succeeded
+      # Remove old xen file after migration succeeded
+      # Note that _RemoveConfigFile silently succeeds if the file is already
+      # deleted, that makes this function idempotent
       try:
         self._RemoveConfigFile(instance.name)
       except EnvironmentError:
         logging.exception("Failure while removing instance config file")
+    else:
+      # Cleanup the most common failure case when the source instance fails
+      # to freeze and remains running renamed:
+      # XM: renamed to 'migrating-${oldname}'
+      # XL: renamed to '${oldname}--migratedaway'
+
+      temp_name, info = None, None
+      for n in ['migrating-' + instance.name, instance.name + '--migratedaway']:
+        info = self.GetInstanceInfo(n, hvparams=instance.hvparams)
+        if info:
+          temp_name = n
+          break
+
+      if info:
+        self._RenameInstance(temp_name, instance.name, instance.hvparams)
 
   def GetMigrationStatus(self, instance):
     """Get the migration status
@@ -1598,7 +1696,8 @@ class XenHvmHypervisor(XenHypervisor):
     else:
       config.write("vnclisten = '%s'\n" % hvp[constants.HV_VNC_BIND_ADDRESS])
 
-    if instance.network_port > constants.VNC_BASE_PORT:
+    if (instance.network_port is not None
+        and instance.network_port > constants.VNC_BASE_PORT):
       display = instance.network_port - constants.VNC_BASE_PORT
       config.write("vncdisplay = %s\n" % display)
       config.write("vncunused = 0\n")
@@ -1609,7 +1708,7 @@ class XenHvmHypervisor(XenHypervisor):
     vnc_pwd_file = hvp[constants.HV_VNC_PASSWORD_FILE]
     try:
       password = utils.ReadFile(vnc_pwd_file)
-    except EnvironmentError, err:
+    except EnvironmentError as err:
       raise errors.HypervisorError("Failed to open VNC password file %s: %s" %
                                    (vnc_pwd_file, err))
 
