@@ -113,7 +113,7 @@ class LogicalVolume(base.BlockDev):
     """
     assert len(pvs_info) > 0
     smallest = min([pv.size for pv in pvs_info])
-    return smallest / (1 + constants.PART_MARGIN + constants.PART_RESERVED)
+    return smallest // (1 + constants.PART_MARGIN + constants.PART_RESERVED)
 
   @staticmethod
   def _ComputeNumPvs(size, pvs_info):
@@ -134,7 +134,7 @@ class LogicalVolume(base.BlockDev):
     """Return a list of empty PVs, by name.
 
     """
-    empty_pvs = filter(objects.LvmPvInfo.IsEmpty, pvs_info)
+    empty_pvs = [pv for pv in pvs_info if objects.LvmPvInfo.IsEmpty(pv)]
     if max_pvs is not None:
       empty_pvs = empty_pvs[:max_pvs]
     return [pv.name for pv in empty_pvs]
@@ -281,7 +281,7 @@ class LogicalVolume(base.BlockDev):
     try:
       info = cls._GetVolumeInfo("pvs", ["pv_name", "vg_name", "pv_free",
                                         "pv_attr", "pv_size", lvfield])
-    except errors.GenericError, err:
+    except errors.GenericError as err:
       logging.error("Can't get PV information: %s", err)
       return None
 
@@ -377,7 +377,7 @@ class LogicalVolume(base.BlockDev):
     try:
       info = cls._GetVolumeInfo("vgs", ["vg_name", "vg_free", "vg_attr",
                                         "vg_size"])
-    except errors.GenericError, err:
+    except errors.GenericError as err:
       logging.error("Can't get VG information: %s", err)
       return None
 
@@ -467,17 +467,17 @@ class LogicalVolume(base.BlockDev):
     try:
       major = int(major)
       minor = int(minor)
-    except (TypeError, ValueError), err:
+    except (TypeError, ValueError) as err:
       base.ThrowError("lvs major/minor cannot be parsed: %s", str(err))
 
     try:
       pe_size = int(float(pe_size))
-    except (TypeError, ValueError), err:
+    except (TypeError, ValueError) as err:
       base.ThrowError("Can't parse vg extent size: %s", err)
 
     try:
       stripes = int(stripes)
-    except (TypeError, ValueError), err:
+    except (TypeError, ValueError) as err:
       base.ThrowError("Can't parse the number of stripes: %s", err)
 
     pv_names = []
@@ -712,7 +712,7 @@ class LogicalVolume(base.BlockDev):
       # amount is in KiB, free_space in MiB
       if amount > free_space * 1024:
         base.ThrowError("Not enough free space to grow %s: %d MiB required,"
-                        " %d available", self.dev_path, amount / 1024,
+                        " %d available", self.dev_path, amount // 1024,
                         free_space)
       # Disk growth doesn't grow the number of spindles, so we must stay within
       # our assigned volumes
@@ -808,7 +808,7 @@ class PersistentBlockDevice(base.BlockDev):
     self.attached = False
     try:
       st = os.stat(self.dev_path)
-    except OSError, err:
+    except OSError as err:
       logging.error("Error stat()'ing %s: %s", self.dev_path, str(err))
       return False
 
@@ -954,7 +954,7 @@ class RADOSBlockDevice(base.BlockDev):
 
     try:
       st = os.stat(self.dev_path)
-    except OSError, err:
+    except OSError as err:
       logging.error("Error stat()'ing %s: %s", self.dev_path, str(err))
       return False
 
@@ -1061,12 +1061,12 @@ class RADOSBlockDevice(base.BlockDev):
     """
     try:
       devices = serializer.LoadJson(output)
-    except ValueError, err:
+    except ValueError as err:
       base.ThrowError("Unable to parse JSON data: %s" % err)
 
     # since ceph mimic the json output changed from dict to list
     if isinstance(devices, dict):
-      devices = devices.values()
+      devices = list(devices.values())
 
     rbd_dev = None
     for d in devices:
