@@ -32,10 +32,12 @@
 """
 
 import logging
-import OpenSSL
 import os
 import time
-from cStringIO import StringIO
+
+from io import StringIO
+
+import OpenSSL
 
 from ganeti import constants
 from ganeti import errors
@@ -74,12 +76,12 @@ def _VerifyCertificateStrong(cert_pem, error_fn,
   try:
     cert = \
       OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, cert_pem)
-  except Exception, err:
+  except Exception as err:
     raise error_fn("(stdin) Unable to load certificate: %s" % err)
 
   try:
     key = OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, cert_pem)
-  except OpenSSL.crypto.Error, err:
+  except OpenSSL.crypto.Error as err:
     raise error_fn("(stdin) Unable to load private key: %s" % err)
 
   # Check certificate with given key; this detects cases where the key given on
@@ -118,7 +120,7 @@ def _VerifyCertificateSoft(cert_pem, error_fn,
   """
   try:
     OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, cert_pem)
-  except OpenSSL.crypto.Error, err:
+  except OpenSSL.crypto.Error as err:
     pass
   else:
     raise error_fn("No private key may be given")
@@ -126,7 +128,7 @@ def _VerifyCertificateSoft(cert_pem, error_fn,
   try:
     cert = \
       OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, cert_pem)
-  except Exception, err:
+  except Exception as err:
     raise errors.X509CertError("(stdin)",
                                "Unable to load certificate: %s" % err)
 
@@ -182,20 +184,42 @@ def VerifyClusterName(data, error_fn, cluster_name_constant,
   return name
 
 
+def VerifyHmac(data, error_fn):
+  """Verifies the presence of the hmac secret.
+
+  @type data: dict
+
+  """
+  hmac = data.get(constants.NDS_HMAC)
+  if not hmac:
+    raise error_fn("Hmac key must be provided")
+
+  return hmac
+
+
 def LoadData(raw, data_check):
   """Parses and verifies input data.
 
   @rtype: dict
 
   """
-  return serializer.LoadAndVerifyJson(raw, data_check)
+  result = None
+  try:
+    result = serializer.LoadAndVerifyJson(raw, data_check)
+    logging.debug("Received data: %s", serializer.DumpJson(result))
+  except Exception as e:
+    logging.warn("Received data is not valid json: %s.", str(raw))
+    raise e
+  return result
 
 
-def GenerateRootSshKeys(error_fn, _suffix="", _homedir_fn=None):
+def GenerateRootSshKeys(key_type, key_bits, error_fn, _suffix="",
+                        _homedir_fn=None):
   """Generates root's SSH keys for this node.
 
   """
-  ssh.InitSSHSetup(error_fn=error_fn, _homedir_fn=_homedir_fn, _suffix=_suffix)
+  ssh.InitSSHSetup(key_type, key_bits, error_fn=error_fn,
+                   _homedir_fn=_homedir_fn, _suffix=_suffix)
 
 
 def GenerateClientCertificate(
