@@ -99,22 +99,18 @@ module Ganeti.Utils
   , frequency
   ) where
 
-import Control.Applicative
 import Control.Concurrent
 import Control.Exception (try, bracket)
 import Control.Monad
 import Control.Monad.Error
+import Control.Monad.Fail (MonadFail)
 import qualified Data.Attoparsec.ByteString as A
 import qualified Data.ByteString.UTF8 as UTF8
 import Data.Char (toUpper, isAlphaNum, isDigit, isSpace)
 import qualified Data.Either as E
 import Data.Function (on)
 import Data.IORef
-#if MIN_VERSION_base(4,8,0)
-import Data.List hiding (isSubsequenceOf)
-#else
 import Data.List
-#endif
 import qualified Data.Map as M
 import Data.Maybe (fromMaybe)
 import qualified Data.Set as S
@@ -232,7 +228,7 @@ if' _    _ y = y
 -- * Parsing utility functions
 
 -- | Parse results from readsPrec.
-parseChoices :: Monad m => String -> String -> [(a, String)] -> m a
+parseChoices :: MonadFail m => String -> String -> [(a, String)] -> m a
 parseChoices _ _ [(v, "")] = return v
 parseChoices name s [(_, e)] =
     fail $ name ++ ": leftover characters when parsing '"
@@ -240,7 +236,7 @@ parseChoices name s [(_, e)] =
 parseChoices name s _ = fail $ name ++ ": cannot parse string '" ++ s ++ "'"
 
 -- | Safe 'read' function returning data encapsulated in a Result.
-tryRead :: (Monad m, Read a) => String -> String -> m a
+tryRead :: (MonadFail m, Read a) => String -> String -> m a
 tryRead name s = parseChoices name s $ reads s
 
 -- | Parse a string using the 'Read' instance.
@@ -278,7 +274,7 @@ printTable lp header rows isnum =
   formatTable (header:rows) isnum
 
 -- | Converts a unit (e.g. m or GB) into a scaling factor.
-parseUnitValue :: (Monad m) => Bool -> String -> m Rational
+parseUnitValue :: (MonadFail m) => Bool -> String -> m Rational
 parseUnitValue noDecimal unit
   -- binary conversions first
   | null unit                     = return 1
@@ -301,7 +297,7 @@ parseUnitValue noDecimal unit
 -- Input must be in the format NUMBER+ SPACE* [UNIT]. If no unit is
 -- specified, it defaults to MiB. Return value is always an integral
 -- value in MiB; if the first argument is True, all kilos are binary.
-parseUnitEx :: (Monad m, Integral a, Read a) => Bool -> String -> m a
+parseUnitEx :: (MonadFail m, Integral a, Read a) => Bool -> String -> m a
 parseUnitEx noDecimal str =
   -- TODO: enhance this by splitting the unit parsing code out and
   -- accepting floating-point numbers
@@ -318,12 +314,12 @@ parseUnitEx noDecimal str =
 -- Input must be in the format NUMBER+ SPACE* [UNIT]. If no unit is
 -- specified, it defaults to MiB. Return value is always an integral
 -- value in MiB.
-parseUnit :: (Monad m, Integral a, Read a) => String -> m a
+parseUnit :: (MonadFail m, Integral a, Read a) => String -> m a
 parseUnit = parseUnitEx False
 
 -- | Tries to extract a number and scale from a given string, taking
 -- all kilos to be binary.
-parseUnitAssumeBinary :: (Monad m, Integral a, Read a) => String -> m a
+parseUnitAssumeBinary :: (MonadFail m, Integral a, Read a) => String -> m a
 parseUnitAssumeBinary = parseUnitEx True
 
 -- | Unwraps a 'Result', exiting the program if it is a 'Bad' value,
@@ -556,7 +552,7 @@ exitIfEmpty _ (x:_) = return x
 exitIfEmpty s []    = exitErr s
 
 -- | Obtain the unique element of a list in an arbitrary monad.
-monadicThe :: (Eq a, Monad m) => String -> [a] -> m a
+monadicThe :: (Eq a, MonadFail m) => String -> [a] -> m a
 monadicThe s [] = fail s
 monadicThe s (x:xs)
   | all (x ==) xs = return x
@@ -818,13 +814,6 @@ ordNub =
         else x : go (S.insert x s) xs
   in go S.empty
 
--- | `isSubsequenceOf a b`: Checks if a is a subsequence of b.
-isSubsequenceOf :: (Eq a) => [a] -> [a] -> Bool
-isSubsequenceOf []    _                    = True
-isSubsequenceOf _     []                   = False
-isSubsequenceOf a@(x:a') (y:b) | x == y    = isSubsequenceOf a' b
-                               | otherwise = isSubsequenceOf a b
-
 {-# ANN frequency "HLint: ignore Use alternative" #-}
 -- | Returns a list of tuples of elements and the number of times they occur
 -- in a list
